package eu.lucazanini.circleview;

import android.app.Activity;
import android.os.Bundle;
import android.util.Log;
import android.view.MotionEvent;
import android.view.View;
import android.view.View.OnTouchListener;
import android.widget.TextView;

public class MainActivity extends Activity {

    private final static String MORE_THAN_TWO_TOUCHES = "I don't handle more than two touches";
    private final static String TAG = MainActivity.class.getName();
    private TouchView touchView;
    private TextView[] xValue, yValue, indexValue, pointerValue, actionValue;

    private void initViews() {
	indexValue = new TextView[2];
	pointerValue = new TextView[2];
	xValue = new TextView[2];
	yValue = new TextView[2];
	actionValue = new TextView[2];

	indexValue[0] = (TextView) findViewById(R.id.indexValue1);
	indexValue[1] = (TextView) findViewById(R.id.indexValue2);

	pointerValue[0] = (TextView) findViewById(R.id.pointerValue1);
	pointerValue[1] = (TextView) findViewById(R.id.pointerValue2);

	xValue[0] = (TextView) findViewById(R.id.xValue1);
	xValue[1] = (TextView) findViewById(R.id.xValue2);

	yValue[0] = (TextView) findViewById(R.id.yValue1);
	yValue[1] = (TextView) findViewById(R.id.yValue2);

	actionValue[0] = (TextView) findViewById(R.id.actionValue1);
	actionValue[1] = (TextView) findViewById(R.id.actionValue2);

	touchView = (TouchView) findViewById(R.id.circle);
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
	super.onCreate(savedInstanceState);

	setContentView(R.layout.activity_main);

	initViews();

	touchView.setOnTouchListener(new OnTouchListener() {

	    @Override
	    public boolean onTouch(View v, MotionEvent event) {

		final int action = event.getActionMasked();
		int pointerIndex, pointerId;
		float x, y;

		switch (action) {
		case MotionEvent.ACTION_DOWN:

		    pointerIndex = event.getActionIndex();
		    pointerId = event.getPointerId(pointerIndex);
		    x = event.getX(pointerIndex);
		    y = event.getY(pointerIndex);
		    setTextViews(pointerId, pointerIndex, x, y, "ACTION_DOWN");
		    moveCircle(pointerId, x, y, action);

		    break;
		case MotionEvent.ACTION_MOVE:
		    for (pointerIndex = 0; pointerIndex < event
			    .getPointerCount(); pointerIndex++) {

			pointerId = event.getPointerId(pointerIndex);
			x = event.getX(pointerIndex);
			y = event.getY(pointerIndex);
			setTextViews(pointerId, pointerIndex, x, y,
				"ACTION_MOVE");
			moveCircle(pointerId, x, y, action);

		    }
		    break;
		case MotionEvent.ACTION_CANCEL:

		    for (pointerIndex = 0; pointerIndex < event
			    .getPointerCount(); pointerIndex++) {

			pointerId = event.getPointerId(pointerIndex);
			setTextViews(pointerId, pointerIndex, "ACTION_CANCEL");
			deleteCircle(pointerId);

		    }
		    break;
		case MotionEvent.ACTION_UP:

		    pointerIndex = event.getActionIndex();
		    pointerId = event.getPointerId(pointerIndex);
		    setTextViews(pointerId, pointerIndex, "ACTION_UP");
		    deleteCircle(pointerId);

		    break;
		case MotionEvent.ACTION_POINTER_DOWN:

		    pointerIndex = event.getActionIndex();
		    pointerId = event.getPointerId(pointerIndex);
		    x = event.getX(pointerIndex);
		    y = event.getY(pointerIndex);
		    setTextViews(pointerId, pointerIndex, x, y,
			    "ACTION_POINTER_DOWN");
		    moveCircle(pointerId, x, y, action);

		    break;
		case MotionEvent.ACTION_POINTER_UP:

		    pointerIndex = event.getActionIndex();
		    pointerId = event.getPointerId(pointerIndex);
		    setTextViews(pointerId, pointerIndex, "ACTION_POINTER_UP");
		    deleteCircle(pointerId);

		    break;
		}
		touchView.invalidate();

		return true;

	    }

	    private void deleteCircle(int pointerId) {

		if (pointerId == 0) {
		    touchView.setCircleOne(0, 0, -1);
		} else if (pointerId == 1) {
		    touchView.setCircleTwo(0, 0, -1);
		} else {
		    Log.d(TAG, MORE_THAN_TWO_TOUCHES);
		}

	    }

	    private void moveCircle(int pointerId, float x, float y, int action) {

		if (pointerId == 0) {
		    touchView.setCircleOne(x, y, action);
		} else if (pointerId == 1) {
		    touchView.setCircleTwo(x, y, action);
		}

	    }

	    private void resetCoordinates(int index) {
		xValue[index].setText("");
		yValue[index].setText("");
	    }

	    private void setCoordinates(int index, float x, float y) {
		xValue[index].setText(String.valueOf(Math.round(x)));
		yValue[index].setText(String.valueOf(Math.round(y)));
	    }

	    private void setTextViews(int pointerId, int pointerIndex, float x,
		    float y, String actionDescription) {
		if (pointerId < 2) {
		    indexValue[pointerId].setText(Integer
			    .toString(pointerIndex));
		    pointerValue[pointerId].setText(Integer.toString(pointerId));
		    actionValue[pointerId].setText(actionDescription);
		    setCoordinates(pointerId, x, y);
		} else {
		    Log.d(TAG, MORE_THAN_TWO_TOUCHES);
		}
	    }

	    private void setTextViews(int pointerId, int pointerIndex,
		    String actionDescription) {
		if (pointerId < 2) {
		    indexValue[pointerId].setText("");
		    pointerValue[pointerId].setText("");
		    actionValue[pointerId].setText(actionDescription);
		    resetCoordinates(pointerId);
		} else {
		    Log.d(TAG, MORE_THAN_TWO_TOUCHES);
		}
	    }
	});

    }
}
